#!/usr/bin/env python
#
# Script to reproduce GDAL bug #4907
# http://trac.osgeo.org/gdal/ticket/4907 
#
# By adjusting the LINE_STEP and PIXEL_STEP parameters below
# one can see how the real output resolution of 
# the warped image (warpedMap.tif) is affected.
# When both parameters are 1 (geolocation arrays have
# the same size as the input bluemarbleEurope.tif), 
# the quality is perfect.
#
# Knut-Frode Dagestad, 29 Nov 2012

import os
from numpy import arange, meshgrid, flipud
import gdal
import osr

map = 'bluemarbleEurope.tif'
dimension = [342, 456]
lonlatfile = 'lonlat.tif'

########################################################
# Adjust these two parameters to test warping quality
########################################################
LINE_STEP = 10
PIXEL_STEP = 10

###########################################
# Make Tiff images with lon and lat
###########################################
lons = arange(0, dimension[0], PIXEL_STEP)
lats = flipud(arange(30, dimension[1], LINE_STEP))
lons, lats = meshgrid(lons, lats)

driver = gdal.GetDriverByName('GTiff')
dst = driver.Create(lonlatfile, lons.shape[1], lons.shape[0], 2, gdal.GDT_Float32)
dst.GetRasterBand(1).WriteArray(lons)
dst.GetRasterBand(2).WriteArray(lats)
dst = None

###########################################
# Create physical VRT file which uses bands 
#   from lonlat.tif as geolocation arrays
###########################################
src = gdal.Open(map)
geolocMap = 'geolocatedMap.vrt'
VRTDriver = gdal.GetDriverByName('VRT')
vrt = VRTDriver.CreateCopy(geolocMap, src)
geolocArray = {}
SRS = osr.SpatialReference()
SRS.ImportFromProj4('+proj=latlong +ellps=WGS84 +datum=WGS84')
geolocArray['SRS'] = SRS.ExportToWkt()
geolocArray['X_DATASET'] = lonlatfile
geolocArray['Y_DATASET'] = lonlatfile
geolocArray['X_BAND'] = '1'
geolocArray['Y_BAND'] = '2'
geolocArray['LINE_OFFSET'] = str(0)
geolocArray['LINE_STEP'] = str(LINE_STEP)
geolocArray['PIXEL_OFFSET'] = str(0)
geolocArray['PIXEL_STEP'] = str(PIXEL_STEP)
vrt.SetMetadata(geolocArray, 'GEOLOCATION')
vrt = None

#############################################################
# Warping the geolocated map, using the geolocation arrays 
#############################################################
os.system('gdalwarp -geoloc ' + geolocMap + ' warpedMap.tif')
os.remove(lonlatfile)
os.remove(geolocMap)
